#
# $Copyright: Copyright (c) 2013 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

use strict;

package Prod;

sub revert_base_version_sys {
    my ($prod,$sys,$pkg,$pkgvers,$mpvers,$force_flag) = @_;
    my ($basevers,$cpic,$pkg_mpvers,$rel);
    $cpic = Obj::cpic();
    $rel=$sys->rel;
    # get the pstamp value of pkg to determine pkg version on Solaris
    $pkg_mpvers = $pkg->mpversion_sys($sys,$force_flag) if ($prod->{padv} =~ /Sol/m);
    $pkgvers = ($pkg_mpvers || $pkgvers);
    # cannot get patch version from disk, hence only check mainpkg version.
    # return mainpkg version if pkg only release
    return $pkgvers if ($cpic->{fromdisk} || $rel->{nopatches});
    # fix for e1989658, there is no need to revert on HPUX since HP is pkg release
    return $pkgvers if ($mpvers || !$pkgvers || $prod->{padv} =~ /HPUX/m);
    $basevers = (($rel->{rp})||($rel->{type} eq "HF")) ? $rel->{basevers}[$#{$rel->{basevers}}] : $rel->{basevers};
    return !EDRu::compvers($pkgvers,$prod->{vers},4) ? ($prod->{basevers} || $basevers) : $pkgvers;
}

sub startprocs {
    my $prod=shift;
    my(@procs,$pkg);
    my $rel = Obj::rel();
    # what about the hypothetical any to many that differ?
    for my $pkgi (@{$prod->allpkgs}) {
        $pkg=$prod->pkg($pkgi);
        if ($pkg->{startprocs}) {
                next if (($rel->{type} eq "HF") && 
                    ($rel->{customized_stoplist}) && 
                    (!EDRu::inarr($pkgi,@{$rel->{ppatch_stopprocs}})));
            push(@procs, @{$pkg->{startprocs}}) if ($pkg->{startprocs});
        }
    }
    return EDRu::arruniq(@procs);
}

sub stopprocs {
    my $prod=shift;
    my ($cpic,@procs,$pkg);
    my $rel = Obj::rel();
    # what about the hypothetical any to many that differ?
    $cpic=Obj::cpic() if Obj::created('CPIC');
    @procs=();
    if($cpic && Cfg::opt(qw(patchupgrade upgrade install uninstall))) {
        for my $pkgi (@{$cpic->{uninstallpkgs_seq}}) {
            $pkg=Pkg->new_pkg($pkgi, $prod->{padv});
            if ($pkg->{stopprocs}) {
                next if (($rel->{type} eq "HF") && 
                    ($rel->{customized_stoplist}) && 
                    (!EDRu::inarr($pkgi,@{$rel->{ppatch_stopprocs}})));
                push(@procs, @{$pkg->{stopprocs}}) if ($pkg->{stopprocs});
            }
        }
        for my $proci (@procs) {
            CPIC::init_cpip_object("Proc::$proci",$prod->{padv})
                unless (Obj::created('Proc', $proci, $prod->{padv}));
        }
        # TODO:  need check upsell install here.
        #        which need stop procs of all pkgs
        return EDRu::arruniq(@procs);
    }
    for my $pkgi (reverse(@{$prod->allpkgs})) {
        $pkg=$prod->pkg($pkgi);
        push(@procs, @{$pkg->{stopprocs}}) if ($pkg->{stopprocs});
    }
    return EDRu::arruniq(@procs);
}


package CPIC;

sub create_cpip_objects {
    my ($cpic,$padv) = @_;
    my ($rel,$patch,$proc,$prevpkgname,$prodv,$pkgi,$pkglist,@pkgs,$pkg,$patchi,$prod,$media_allpkgs);
    if (!EDRu::inarr($padv, @{EDR::padvs()}, 'Common')) {
        # import anything to operate the rest of the commands OK
        # unsupported padvs will eventually die later
        Msg::log("create_cpip_objects: $padv is not a supported platform");
        return;
    } elsif ($Obj::pool{"Rel\::$cpic->{release}\::$padv"}) {
        #Msg::log("$cpic->{release} $padv CPIP objects already created");
        return $cpic->rel($padv);
    }
    #Msg::log("Creating $cpic->{release}::$padv CPIP objects");
    $rel=$cpic->{rel}{$padv}=init_cpip_object("Rel::$cpic->{release}",$padv);
    push(@pkgs, @{$rel->allpkgs});
    for my $prodv (@{$rel->{prods}}) {
        $prod=init_cpip_object("Prod::$prodv",$padv);
        $prod->{release}=$cpic->{release};
        $prod->set_pkgs;
        push(@pkgs, @{$prod->allpkgs_on_media});
        for my $subprodv(@{$prod->{subprods}}) {
            my $subprod=init_cpip_object("Prod::$subprodv",$padv);
            $subprod->{release}=$cpic->{release};
            push(@pkgs, @{$subprod->allpkgs_on_media});
        }
    }
    $pkglist=EDRu::arruniq(@pkgs);
    for my $pkgi (@{$pkglist}) {
        $pkg=init_cpip_object("Pkg::$pkgi",$padv);
        for my $proc (@{$pkg->allprocs}) { init_cpip_object("Proc::$proc",$padv); }
        for my $prevpkgname (@{$pkg->{previouspkgnames}}) {
            Pkg->new_pkg($prevpkgname, $padv);
        }
    }
    $rel->define_patches()      unless ($rel->{type} =~ /HF/);
    $rel->define_hotfixes();
    for my $pkgi (@{$pkglist}) {
        $pkg=Obj::pkg($pkgi, $padv);
        for my $patchi (@{$pkg->patches_allvers}) {
            # Patches are not built-in defined in CPIP, create it.
            $patch=new_object("Patch::$patchi",$padv);
            $patch->{pkg}=$pkgi;
        }
    }
    $cpic->create_eeb_objects($rel,$padv) if ($rel->{reli}=~/^EEB/);
    $rel->adjust_pkglist();
    return $rel;
}


=for text
## [[   change block 20130528.chawley2
Merged features from v61 branch (bbai) which use disparate methods for 
comparison.  For hotfixes, we don't want a product aggregate, just the 
comparable package version.

=cut

sub verify_pkg_version_sys {
    my $cpic = shift;
    my $sys = shift;
    my ($cv, $msg, $padv, $patch, @patchpkgs, $pkg, $pkgname, $pvers, $rel);
    $rel = $cpic->rel;
    # padv invariant, so outside of main loop
    $padv = $sys->padv;
    # Get names of packages which carry ppatches information
    @patchpkgs = map{ $Obj::pool{$_}->{pkg} } grep{ exists $Obj::pool{$_}->{ppatches} } grep{/^Pkgs::/} sort keys %Obj::pool;
    for $pkgname (@patchpkgs) {
        $pkg = $sys->pkg($pkgname);
        #$padv = $sys->padv;
        for my $patchi ( @{$pkg->patches_sys($sys)} ) {
            $patch = $sys->patch($patchi);
            if ($sys->aix()) {
                # AIX patch check is essentially a pkg check
                $pvers = $pkg->version_sys($sys);
                $cv = EDRu::compvers($patch->{vers}, $pvers);
                if ($cv == 2) {
                    $msg = Msg::new("The higher version of $pkgname($pvers) is already installed on $sys->{sys}");
                    $sys->push_error($msg);
                    last;
                } elsif ($cv == 0) {
                    $msg = Msg::new("$pkgname($pvers) is already installed on $sys->{sys}");
                    $sys->push_note($msg);
                }
            } else {
                $pvers = $padv->patch_version_sys($sys, $patch);
                $cv = EDRu::compvers($patch->{vers}, $pvers, 4);
                if ($cv == 2) {
                    $msg = Msg::new("The higher version of $patch->{patchname}($pvers) is already installed on $sys->{sys}");
                    $sys->push_error($msg);
                    last;
                }
                if ($cv == 0) {
                    $msg = Msg::new("$patch->{patchname}($pvers) is already installed on $sys->{sys}");
                    $sys->push_note($msg);
                }
            }
        }
    }
}

sub verify_prod_version_sys {
    my $cpic = shift;
    my $sys = shift;
    my ($rel, $cv, $ivers, $msg, $prodname, $prod);
    $prod=$cpic->prod;
    $rel = $cpic->rel;
    $ivers=$prod->version_sys($sys);
    $cv=EDRu::compvers($ivers,$prod->{vers},4)  if ($ivers);
    if (!($sys->{errors}) && $cv && Cfg::opt(qw(upgrade patchupgrade hotfixupgrade))) {
        $prodname= $sys->{dbed_prod} || $prod->{abbr};
        $prodname= $prod->{upgrade_from}        if (defined $prod->{upgrade_from});
        $ivers = $prod->version_with_extra_mainpkgs_sys($sys);
        $msg=Msg::new("$prodname version $ivers will be upgraded to $prod->{abbr} version $prod->{vers} on $sys->{sys}");
        $sys->push_note($msg)   unless (EDRu::inarr($prodname,@{$rel->{ru_noupgrademsg_prod}}) && 
                        (Cfg::opt('upgrade_nonkernelpkgs') || $rel->determine_rolling_upgrade($cpic->{prod})==2));
    }
}


sub verify_all_systems {
    my $cpic=shift;
    my ($ivers,$msg,$sys,$prod,$prodname,$rel,$cv);
    return 1 unless ($cpic->{prod});
    $prod=$cpic->prod;
    $rel=$cpic->rel;
    $cpic->cli_set_title();

    for my $sys (@{$cpic->{systems}}) {
        # Note: how should this handle some systems having stop_checks set?
        next            if ($sys->{stop_checks});
        $ivers = $prod->version_sys($sys);
        if ($rel->{type} eq 'HF') {
            # hotfix only needs to check pkg version
            $cpic->verify_pkg_version_sys($sys);
        } else {
            $cpic->verify_prod_version_sys($sys);
        }
    }
    return 1;
}

=for text
## ]]   change block 20130528.chawley2
## [[   change block 20130212.chawley2
The subroutines copy_cpi_sys() and create_task_script_sys() which follow
are based on their namesakes in F<CPIC/v60/CPIC/install.pm>.

=cut

# this whole thing is probably going away, but saving to compare the two
# upper copy works, this is merge from final 5.1SP1
#
# Copy the CPI libraries to /opt/VRTSperl/lib/site_perl/TRAINVERS
#  - 5.0 copies its base libraries in place 
#  - Install scripts are generated & copied in completion
# Enhanced to copy hotfix-specific modules to  .../site_perl/CPIC/ .
# 
sub copy_cpi_sys {
    my ($cpic,$sys) = @_;
    my ($rootpath,$prodx,@allprods,$rel,$vxgettext_dir,$installscript_prod,$bmcdomaindir,$dir,$mod,$scripts,$installscript_name,$prodname,$prods,$cpidir,$installscript,$prod,$localsys,$patchrelease_file);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $cpic=Obj::cpic()   unless (ref($cpic) eq 'CPIC');
    return 1    if (Cfg::opt('makeresponsefile'));
    return ''   if ($cpic->{prod} && $sys->prod->{nocpionsys});
    $localsys = $cpic->localsys;
    $rel = $sys->rel;
    $rel->copy_cpi_sys($sys);           # for extras
    if ($cpic->{prod}) {
        $sys->prod->copy_cpi_sys($sys); # for product binaries and other scripts
    }

    # condition for -copyinstallscripts option (product not defined yet)
    $prods = ($cpic->{prod}) ? [ $cpic->{prod} ] : $rel->{prods};

    $rootpath = Cfg::opt('rootpath');
    $cpidir = $rootpath.$cpic->{perlpath}.'/lib/site_perl/'.$cpic->{release};

    # copy CPI libraries
    $bmcdomaindir = $cpic->{mediapath}.'/scripts';
    for my $dir ('CPIC','CPIP','EDR') {
        next unless (-e "$bmcdomaindir/$dir");
        $sys->mkdir("$cpidir/$dir");
        $localsys->copy_to_sys($sys, "$bmcdomaindir/$dir/*", "$cpidir/$dir");
    }
    if ($rel->{type} eq "P" || $rel->{type} eq "HF") {
        my $dir = "CPIC";
        my $msg = Msg::new("copy modules for $rel->{ppatchname} type=$rel->{type} to $cpidir/$dir");
        $msg->log;
        $sys->mkdir("$cpidir/$dir");            # okay if it already exists
        $localsys->copy_to_sys($sys, "$bmcdomaindir/*.pm", "$cpidir/$dir");
    }

    # copy CPI message catalogs
    $vxgettext_dir = "$bmcdomaindir/bin";
    # Make sure this image includes messages and a way to decode them.
    if (-x $vxgettext_dir && -d "$bmcdomaindir/messages") {
        $localsys->copy_to_sys($sys, "$vxgettext_dir", "$cpidir");
        $sys->chmod("$cpidir/bin", "+x");
        # We aren't the only player, so don't trash existing messages, ok?
        #- $sys->rm("$cpidir/messages");
        $sys->mkdir("$cpidir/messages");
        $localsys->copy_to_sys($sys, "$bmcdomaindir/messages/*", "$cpidir/messages");
        # FIX: need to sort out unique directories per train as each family
        # ^^^  will have its own bmcmap file
    }

    # copy CPI installer scripts
    $sys->mkdir("$rootpath$cpic->{installpath}");
    $sys->createfile("$rootpath$cpic->{installpath}/.cpi5");

    if (Cfg::opt('upgrade') && (defined $rel->{obsolete_scripts})){
        $scripts=join(' ', @{$rel->{obsolete_scripts}});
        unless ($scripts =~ /^\s*$/m) {
            $sys->cmd("cd $rootpath$cpic->{installpath}; _cmd_rmr $scripts");
        }
    }
    if (!$rel->{nopkgs}) {
        for my $prodname (@$prods) {
            $prod=$sys->prod($prodname);
            @allprods=($prodname);
            push(@allprods,@{$prod->{subprods}}) if ($cpic->{prod} && $prod->{subprods});
            for my $prodx (@allprods) {
                $prod=$sys->prod($prodx);

                # Copy product binaries and other scripts
                $prod->copy_cpi_sys($sys);
                next    if ($prod->{nocpionsys});
                # Copy other modules
                for my $mod (@{$prod->{cpimodules}}) {
                    $localsys->copy_to_sys($sys, "$bmcdomaindir/$mod", $cpidir);
                }
                # Copy task scripts
                $installscript_prod = $prod->{installscript_prod} || $prodx;
                $installscript_name = lc($prod->{installscript_name} || $prod->{prod});
                $installscript_name=~s/\s+//m;
                $installscript = "$rootpath$cpic->{installpath}/install" . $installscript_name;
                $cpic->create_task_script_sys($sys,'configure',$installscript_prod,$installscript) if (EDRu::inarr('configure',@{$rel->{args_task}}));
                $installscript = "$rootpath$cpic->{installpath}/uninstall" . $installscript_name;
                $cpic->create_task_script_sys($sys,'uninstall',$installscript_prod,$installscript) if (EDRu::inarr('uninstall',@{$rel->{args_task}}));
            }
        }
    } else {
        unless (Padv::plat($cpic->{padv}) eq "Linux") {
            $installscript_prod = '';
            $installscript_name = $rel->{ppatchname};
            $installscript = "$rootpath/opt/VRTS/install/uninstall" . $installscript_name;
            $cpic->create_task_script_sys($sys,'rollback',$installscript_prod,$installscript);
        }
    }
    $sys->chown_root("$rootpath$cpic->{installpath}");
    return;
}


sub create_task_script_sys {
    my ($cpic,$sys,$task,$prodname,$filename) = @_;
    my ($script,$rel,$edr,$padvs);
    $rel=$sys->rel;
    $edr=Obj::edr();
    $padvs=join(' ',@{$edr->{padvs}});
    $script=<<'SCRIPT';
#!/bin/sh

#
# Copyright (c) 2010 Symantec Corporation. All rights reserved.
# Symantec and the Symantec Logo are trademarks or registered trademarks
# of Symantec Corporation or its affiliates in the U.S. and other countries.
# Other names may be trademarks of their respective owners.
#
# The Licensed Software and Documentation are deemed to be "commercial
# computer software" and "commercial computer software documentation"
# as defined in FAR Sections 12.212 and DFARS Section 227.7202.
#

PDIR=/opt/VRTSperl
if [ -x "$PDIR/bin/perl" ]; then : ok; else
    echo "$PDIR/bin/perl not found or not executable"
    echo "Cannot find perl to execute $0"
    exit 1
fi

SDIR=/opt/VRTSperl/lib/site_perl/RELEASE
PERLVERS=`$PDIR/bin/perl -MConfig -e 'print \$Config{api_versionstring},"\n"'`
PERLFOR=`$PDIR/bin/perl -MConfig -e 'print \$Config{archname},"\n"'`

IOPT="-I$PDIR/lib/$PERLVERS/$PERLFOR -I$PDIR/lib/$PERLVERS -I$PDIR/lib/site_perl/$PERLVERS/$PERLFOR -I$PDIR/lib/site_perl/$PERLVERS -I$PDIR/lib/site_perl"
IOPT="-I$SDIR/EDR -I$SDIR/CPIC -I$SDIR/CPIP $IOPT"

eval 'exec $PDIR/bin/perl -x -S $IOPT $0 ${1+"$@"}'
        if 0;
#!$PDIR/bin/perl

use RELEASE;
use PATCHNAME;
use Hotfix;

CPIC::main( "RELEASE",
            "PADV",
            "",
            "TASK",
            "PROD");
SCRIPT

    # Transform (only) whole word tokens to their applicable values.
    $script=~s/\bRELEASE\b/$cpic->{release}/mg;
    # in future, p-patch will be called hotfix - handle similarly
    if ($rel->{type} eq "HF" || $rel->{type} eq "P") {
        $script=~s/\bPATCHNAME\b/$rel->{ppatchname}/mg;
    } else {
        $script=~s/^use PATCHNAME;//mg;
    } 
    $script=~s/\bPADV\b/$padvs/mg;
    $script=~s/\bTASK\b/$task/mg;
    $script=~s/\bPROD\b/$prodname/mg;

    $sys->cmd("_cmd_rmr $filename");
    $sys->writefile($script, $filename);
    $sys->chmod($filename, '+x');
    return ''           if (EDR::cmdexit());
    return 1;
}

=for text
## ]]   change block 20130212.chawley2

=cut

sub prod_version_sys {
    my ($cpic,$sys)=@_;
    my ($prod,$prodname,$padv,$ivers,$msg,$opt,$cv,$installedprod,$rel,$noconfig_flag,$prior_rp_title);
    my ($basevers,$baseversi,$basevers_flag,$nums);
    # noconfig_flag is related with SF or VCS partial upgrade out of installed SFHA
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $rel=$sys->rel;
    $noconfig_flag = ($sys->{noconfig_sf} || $sys->{noconfig_vcs});
    return 1 if ($sys->{stop_checks});
    return 1 if ($rel->{lp});
    if ($cpic->{prod}) {
        $prod=$sys->prod;
        $prodname=$prod->{abbr};
        $ivers=$prod->version_sys($sys);
    } else {
        $prodname = 'Any product';
    }
    if (($rel->{type} eq "RP") || ($rel->{type} eq "HF")) {
        $nums = @{$rel->{basevers}} - 1;
        $basevers = $rel->{basevers}[$nums];
    }
    $cv=EDRu::compvers($ivers,$prod->{vers},4)  if ($ivers);

=pod

    ###################     check whether                check whether 
    # task                product installed             current version
    #=======================================================================
    # install                   X                            X
    # upgrade                   V                            X
    # patchupgrade              V                            X
    # uninstall                 V                            V
    # configure                 V                            V
    # start                     V                            V
    # stop                      V                            V
    # license                   V                            V
    # precheck                  X                            X
    #=======================================================================

=cut

    if (Cfg::opt(qw(uninstall rollback configure start stop startstoptest postcheck license))) {
        if (!$ivers) {
            $msg=Msg::new("$prodname $prod->{vers} does not appear to be installed on $sys->{sys}");
            if (Cfg::opt(qw(uninstall license))) {
                $sys->push_warning($msg);
            } else {
                $sys->push_error($msg) unless (Cfg::opt('install') && Cfg::opt('configure'));
                return '';
            }
        } elsif ((${$sys->{upgradeprod}}[0] ne "") && (Cfg::opt("rollback")) && $cv) {
          # No warning message for RP as customers many times have only partial RP installed
          #$msg=Msg::new("$rel->{titlevers} does not appear to be installed on $sys->{sys}");
          #$sys->push_warning($msg);
        } elsif ((Cfg::opt("rollback")) && $sys->{prior_rp}) {
            $prior_rp_title = $rel->{prior_rp_title}{$sys->{prior_rp}};
            $msg=Msg::new("$sys->{sys} will be rolled back to $prior_rp_title release");
            $sys->push_warning($msg);
        }
    } elsif (!$ivers) {
        # install/upgrade/patchupgrade/precheck below :
        # not installed
        if (Cfg::opt(qw(patchupgrade)) && (${$sys->{upgradeprod}}[0] eq '')) {
            return 1 if (Cfg::opt(qw(upgrade patchupgrade)) && $sys->{skip_versioncheck} && $cpic->{prod});
            $msg=Msg::new("$prodname does not appear to be installed on $sys->{sys}");
            $sys->push_stop_checks_error($msg);
            return '';
        } elsif (${$sys->{upgradeprod}}[0] ne '') {
            # cross stack upgrade/install
            $installedprod=$sys->prod(${$sys->{upgradeprod}}[0]);
            $ivers=$installedprod->version_sys($sys);
            $cv=EDRu::compvers($ivers,$installedprod->{vers},4);
            if ($cv==2) {
                if (Cfg::opt('upgrade')) {
                    if (!$installedprod->upgradeable_version_sys($sys)) {
                        $msg=Msg::new("Upgrade from $installedprod->{abbr} $ivers to $prodname $prod->{vers} is not supported on $sys->{sys}");
                        $sys->push_error($msg);
                        return '';
                    }
                    if ($installedprod->zru_version_sys($sys)) {
                        $sys->set_value('zru_supported',1);
                    } else {
                        $sys->set_value('zru_supported',0);
                    }
                }
            } elsif ($cv==1) {
                # more recent version is installed
                $msg=Msg::new("A more recent version of $installedprod->{abbr}, $ivers, is already installed on $sys->{sys}");
                $sys->push_warning($msg);
            }
        } elsif (Cfg::opt(qw(install))) {
            return 1;
        }
    } elsif ($cv==0) {
        # Do not show this msg if P-patch
        return 1        if ($rel->{type} eq "HF");
        $msg=Msg::new("$prodname version $ivers is already installed on $sys->{sys}");
        # When product already installed and user is performing RU phase 2.
        # Now RU's precheck won't call $cpic->prod_version_sys
        # And the option 'upgrade_nonkernelpkgs' will be determined after user confirmed cluster info.
        if (Cfg::opt('upgrade_nonkernelpkgs')) {
            $sys->set_value('zru_supported',1);
            return 1;
        }
        if (!$noconfig_flag) {
            if (Cfg::opt('upgrade')) {
                $sys->push_error($msg);
                return '';
            } else {
                $sys->push_warning($msg);
            }
        } else {
            if ($sys->{prod_to_upgrade} && $sys->{prod_to_upgrade}->zru_version_sys($sys)) {
                $sys->set_value('zru_supported',1);
            } else {
                $sys->set_value('zru_supported',0);
            }
        }
    } elsif (Cfg::opt("patchupgrade") && $rel->{nopkgs}) {
        # patchonly release version check
        # Match if basevers
        for $baseversi (@{$rel->{basevers}}) {
            if (!EDRu::compvers($ivers,$baseversi,3)) {
                $basevers_flag = 1;
            }
        }
        unless ($basevers_flag) {
            $msg=Msg::new("$prodname $basevers does not appear to be installed on $sys->{sys}");
            $sys->push_stop_checks_error($msg);
        }
    } elsif ($cv==1) {
        # more recent version is installed
        $msg=Msg::new("A more recent version of $prodname, $ivers, is already installed on $sys->{sys}");
        $sys->push_warning($msg);
    } elsif ((Cfg::opt(qw(responsefile))) &&
        (!$prod->{responsefileupgradeok})) {
        # prior version installed, can't upgrade w responsefile
        # questions being asked is almost inevitable
        # installonly may need to run start/stop/config routines to upgrade
        $opt = (Cfg::opt('responsefile')) ? '-responsefile' : '-install';
        $msg=Msg::new("$prodname version $ivers is installed on $sys->{sys}.  Upgrading $prodname is not supported using the $opt option");
        $sys->push_error($msg);
        return '';
    } else {
        # Code to check if upgrade from current version is supported or not
        # is yet to be added (UGVERS)
        return ''       if (!$sys->rel->upgradevers_sys($sys));
        if ($prod->zru_version_sys($sys)) {
            $sys->set_value('zru_supported',1);
        } else {
            $sys->set_value('zru_supported',0);
        }
    }
    return 1;
}

sub cli_exit_file {
    my ($ayn,$cfg,$cpic,$edr,$eprod,$esystems,$etask);
    my ($prod,$prodi,$msg,$exitfile,$help,$io,$line,$opt,$opts,$localsys);

    return '' if (Cfg::opt(qw(responsefile makeresponsefile)));
    $cpic=shift;
    $edr=Obj::edr();
    $cfg=Obj::cfg();
    $localsys=$edr->{localsys};
    $esystems = $eprod = $etask = '';

    $exitfile = $edr->{logpath} . "/exitfile-" . $edr->{release} . "-*";
    $edr->{exitpath} = $localsys->path($exitfile);
    $exitfile = $localsys->lsfile($edr->{exitpath});
    $edr->{exitfile}=$exitfile='noexitfile' if ((Cfg::opt(qw(security addnode fencing))) || ($ENV{NOEXITFILE}));
    if (-f $exitfile) {
        $io = EDRu::readfile($exitfile);
        for my $line (split(/\n/, $io)) {
            next if ($line =~ /^\s*#/);
            if (!$esystems && ($line =~ /CFG{systems}/mx)) {
                $line =~ s/\$CFG{systems}=\[\ qw\(//m;
                $line =~ s/\)\ \];//m;
                $esystems = $line;
            }
            if (!$eprod && ($line =~ /CFG{prod}/m)) {
                $prodi = $1 if($line =~ /=\s*["]?(\w+)/mx);
                $prod = $cpic->prod($prodi);
                $eprod = $prod->{abbr};
                #$line =~ s/\$CFG{prod}=\"//;
                #$line =~ s/\d+\";//;
                #$eprod = $line;
            }
            if (!$etask && ($line =~ /{opt}{(install|patchupgrade|upgrade|uninstall|configure|precheck|license|start|stop)}/mx)) {
                $line =~ s/\$CFG{opt}{//mx;
                $line =~ s/}=1;//m;
                $etask = $line;
            }
        }
        return ""       unless ($etask && $esystems);
        if (($etask ne 'upgrade') && ($eprod eq "")) {
            return "";
        }

        # show exitfile prompt only if at least task is selected
        $msg=Msg::new("The installer has discovered an existing installer process. The process exited while performing $etask of $eprod on $esystems. Do you want to resume this process?");
        $help=Msg::new("This message indicates that a previous installer instance failed or exited incorrectly.\nThe message indicates the following information:\n- whether the installer was performing an installation or an upgrade\n- which Veritas product was selected\n- the system names\n\n If you have corrected the issues which caused the installer to exit, you can resume the process. Select y at the prompt. The installer continues the selected task from the point when the installation or upgrade failed.\n If you are performing a different task or do not wish to restart the previous installer process, select n at the prompt");
        $ayn=$msg->ayny($help, 0);
        Msg::n();
        if ($ayn eq 'Y') {
            $edr->{exitfile}=$exitfile;
            Cfg::set_opt('responsefile',$exitfile);
            our %CFG;
            require($cfg->{opt}{responsefile});
            $cpic->set_task($etask);
            $opts=$cfg->{opt};
            %$cfg = (%$cfg, %CFG);
            # merging the hashes with the line above doesn't merge $cfg{opt} correctly
            for my $opt (keys %$opts) { $cfg->{opt}{$opt}=$$opts{$opt}; }
        } else {
            $localsys->rm("$edr->{logpath}/exitfile-*");
        }
    }
    return '';
}


package Rel;

sub check_update {
    my ($rel) = @_;
    my ($sort_status,$prod_upd_cv, $cpi_upd_cv, $lv_cb, $prod_upd_lv_cb, $cpi_upd_lv_cb, $lv_mj, $prod_upd_lv_mj, $cpi_upd_lv_mj);
    my ($cpic,$prod,$prodi,$prodname,$relvers,$prodvers,$msg,$patchname,$basevers);
    return if (! Cfg::opt(qw(install upgrade patchupgrade)));
    return if (Cfg::opt(qw(makeresponsefile)));
    $cpic=Obj::cpic();
    return if (! $cpic->{prod});
    return if ($cpic->{eebi});
    $prod = $cpic->prod();
    $prodname = $prod->{name};
    $prodi = lc($prod->{prod});
    $relvers = $rel->{titlevers};
    if ($rel->{type} eq "HF") {
        $relvers =~ s/\sP\d+$//;
    }
    $prodvers = $relvers;
    $prodvers =~ s/\s//g;
    $basevers = $prodvers;
    $basevers =~ s/^([0-9\.]*).*$/$1/;
    $msg = Msg::new("Checking online updates for $prodname $relvers");
    $msg->bold;
    ($sort_status,$prod_upd_cv, $cpi_upd_cv, $lv_cb, $prod_upd_lv_cb, $cpi_upd_lv_cb, $lv_mj, $prod_upd_lv_mj, $cpi_upd_lv_mj) = $cpic->get_releases_patches_via_sort_webservice($prodi, $cpic->{padv}, $prodvers);
    $msg = "\n";
    if ($sort_status) {
        # current installed version
        if ($prod_upd_cv && @$prod_upd_cv || $cpi_upd_cv && @$cpi_upd_cv) {
            $msg .= Msg::new("\tAvailable updates for $prodname $relvers:\n")->{msg};
            for my $patch (@$prod_upd_cv) {
                $patchname = $patch->{release_name};
                $msg .= Msg::new("\t    $patchname\n")->{msg};
            }
            for my $patch (@$cpi_upd_cv) {
                $patchname = $patch->{release_name};
                $msg .= Msg::new("\t    (Installer) $patchname\n")->{msg};
            }
        } else {
            $msg .= Msg::new("\tNo updates available for $prodname $relvers\n")->{msg};
        }
        # latest version in current release branch.
        if ($lv_cb) {
            $msg .= "\n";
            $msg .= Msg::new("\tThe latest version of $prodname $basevers release is: $lv_cb\n")->{msg};
            if ($prod_upd_lv_cb && @$prod_upd_lv_cb || $cpi_upd_lv_cb && @$cpi_upd_lv_cb) {
                $msg .= Msg::new("\t  Available updates for $prodname $lv_cb:\n")->{msg};
                for my $patch (@$prod_upd_lv_cb) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    $patchname\n")->{msg};
                }
                for my $patch (@$cpi_upd_lv_cb) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    (Installer) $patchname\n")->{msg};
                }
            } else {
                $msg .= Msg::new("\t  No updates available for $prodname $lv_cb\n")->{msg};
            }
        }
        # latest major release version
        if ($lv_mj) {
            $msg .= "\n";
            $msg .= Msg::new("\tThe latest version of $prodname is: $lv_mj\n")->{msg};
            if ($prod_upd_lv_mj && @$prod_upd_lv_mj || $cpi_upd_lv_mj && @$cpi_upd_lv_mj) {
                $msg .= Msg::new("\t  Available updates for $prodname $lv_mj:\n")->{msg};
                for my $patch (@$prod_upd_lv_mj) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    $patchname\n")->{msg};
                }
                for my $patch (@$cpi_upd_lv_mj) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    (Installer) $patchname\n")->{msg};
                }
            } else {
                $msg .= Msg::new("\t  No updates available for $prodname $lv_mj\n")->{msg};
            }
        }

        $msg .= "\n";
        $msg .= Msg::new("\tPlease visit https://sort.symantec.com for more information.\n")->{msg};
    } else {
        $msg .= Msg::new("\tAttempted to connect to https://sort.symantec.com to check for product updates, but connection failed.\n")->{msg};
        $msg .= Msg::new("\tPlease visit https://sort.symantec.com to check for available product updates and information.\n")->{msg};
    }
    Msg::print($msg);
    if (Obj::webui()) {
        my $web = Obj::web();
#        $msg =~ s/(https\:\/\/\S+)\s+/<a href="$1">$1<\/a>/g;
        $msg =~ s/^\t\s{4}/&nbsp;&nbsp;&nbsp;&nbsp;/mg;
        $msg =~ s/^\t\s{2}/&nbsp;&nbsp;/mg;
        $msg =~ s/\t//g;
        $msg =~ s/\n/\\n/g;
        $web->web_script_form('check_update',$msg);
    }
    return;
}

1;

=head2 LAST UPDATED

 2013-06-06 18:59	- chawley2
	Disable $rel->define_patches whenever we are playing hotfix.

=cut

