#
# $Copyright: Copyright (c) 2013 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $ 
#

use strict;

package Rel::UXRT601::Common;

sub init_common {
    my ($rel) = @_;
    my $mediapath;

    $rel->{verify_media}=1;     # hotfix layout is stable enough 
    $rel->{reli}='UXRT';
    $rel->{reltitle}=Msg::new("Storage Foundation and HA Solutions")->{msg};
    $rel->{vers}='6.0.300.100';
    $rel->{basevers}=[ "6.0.100" ];
    $rel->{gavers}='6.0.100.000';
    $rel->{titlevers}='6.0.3 P1';
    $rel->{nopkgs}=1;
    $rel->{ppatchname}="LLT603P1";
    $rel->{type}="HF";
    $rel->{patchtype}="ppatches";
    $rel->{customized_stoplist}=0;
    $rel->{ppatch_stopprocs}=[ qw(VRTSllt60) ];
    $rel->{currentrelease}="UXRT603";

    $rel->{build_state}='_BUILD_STATE_';
    $rel->{build_sprint}='_BUILD_SPRINT_';
    $rel->{build_date}='2013-04-15';

    $rel->{sort_upload_config_url}='https://telemetrics.symantec.com/data/uploader/uxrt603upload.conf';

    $rel->{prods}=[ qw(FS60 DMP60 VM60 VCS60 SF60 SFHA60 SFCFSHA60 SFCFSRAC60 SVS60 SFSYBASECE60 SFRAC60) ];
    $rel->{pkgsetprods}=[ qw(SFRAC60 SFSYBASECE60 SVS60 SFCFSHA60 SFHA60 SF60 VCS60 DMP60) ];
    #$rel->{menuprods}=EDRu::arrdel($rel->{prods}, qw(FS60 VM60 SFCFSRAC60));
    $rel->{menuprods}=[ qw(DMP60 VCS60 SF60 SFHA60 SFCFSHA60 SVS60 SFSYBASECE60 SFRAC60) ];
    $rel->{upgradeprods} = [ qw(SFRAC60 SFSYBASECE60 SFCFSHA60 SFHA60 SF60 VCS60 VM60 DMP60 FS60) ];
    $rel->{allpkgs}=[ qw(VRTSperl514 VRTSvlic32 VRTSsfcpi601 VRTSspt60) ];
    $rel->{minpkgs}=[ qw(VRTSperl514 VRTSvlic32 VRTSsfcpi601) ];
    $rel->{recpkgs}=[ qw(VRTSperl514 VRTSvlic32 VRTSsfcpi601 VRTSspt60) ];
    ##reinstated  TODO: commment out temporarily since cpi patch is not ready
    $rel->{cpipkg}='VRTSsfcpi601';

    $rel->{ru_prod}=[ qw(SFRAC60 SVS60 SFSYBASECE60 SFCFSHA60 SFHA60 VCS60) ];
    $rel->{ru_unkernelpkgs}=[ qw(VRTSvcs60 VRTSvcsag60 VRTSvcsea60 VRTScavf60) ];
    $rel->{ru_noupgrademsg_prod}=[ qw(SFRAC) ];
    $rel->{ru_version}='5.1';
    $rel->{bucket_version}='5.1';
    $rel->{pkgset_version}='5.1';

    $rel->{args_task}=[ qw(patchupgrade) ]; 
    my $padv=Obj::padv($rel->{padv});
    # FIX e3189618:  'Cannot find patchpath for padv RHEL5x8664' (any Linux).
    # Only have common version for HF installer, so fold naming logic in here.
    #$padv->{patchpath} = (Padv::plat($rel->{padv}) eq "Linux") ? "rpms" : "patches";
    $padv->{patchpath} = "patches";             # inconsistent patch, surprise
}

sub cli_installer_menu {
    my ($rel) = @_;
    my ($cfg,$cpic,$prod,$msg,$mtask,$mprod,$mmode,$moptions,$task,$cprod);
    my ($edr, $args_hash, @options);
    return '' if (Cfg::opt('responsefile') && (EDR::get('exitfile') eq ''));

    $edr=Obj::edr();
    $cpic=Obj::cpic();
    $cfg=Obj::cfg();
    $mtask='';
    $mprod='';
    $mmode='';
    $moptions=[];

    set_menu_msgs();
    sleep 3;
    $task=$cpic->{task};
    for my $mtask (@{$rel->{args_task}}) {
        if (Cfg::opt("$mtask")) {
            $task=$mtask;
            last;
        }
    }
    $cprod=$cfg->{prod}; # for the exitfile
    $cprod||=$cpic->{orig_prod};

    while (1) {
        EDR::reset_errors_warnings();

        if ($task) {
            $mtask=lc($task);
        } else {
            $mtask||=$rel->task_menu($mtask);
            if (EDR::getmsgkey($mtask,'back')) {
                $mtask='';
                next;
            } elsif ($mtask eq 'help') {
                $rel->task_menu_help();
                $mtask='';
                next;
            }
        }
        $cpic->set_task($mtask);
        $cpic->{task}=$mtask;
        if(Cfg::opt('upgrade') && (!$task)){
            my $upgrade_task=$rel->upgrade_menu();
            if(EDR::getmsgkey($upgrade_task,'back')){
                $cfg->{opt}{$mtask}=0;
                $mprod='';
                $mtask='';
                next;
            } elsif($upgrade_task){
                Cfg::set_opt($upgrade_task);
                Msg::n();
                # According to e2789316, -makeresponsefile could not co-exists with rollingupgrade options
                # Check if -makeresponsefile options exist when user select rolling upgrade task from task menu
                @options=keys %{Obj::cfg()->{opt}};
                $args_hash->{options}=EDRu::arrdel(\@options, @{$edr->{arguments}->{args_opt}});
                $rel->process_ru_arg($args_hash) if(Cfg::opt('rolling_upgrade'));
            }
            last;
        }
        last if Cfg::opt('upgrade');
        return "" if ($rel->{nopkgs});  # No product menu for P patch install

        if (!$cprod) {
            $mprod||=$rel->prod_menu($mtask,$mprod);
        } else {
            $mprod||=$cprod; # for exitfile
        }
        if (EDR::getmsgkey($mprod,'back')) {
            delete $cfg->{opt}{$mtask};
            $mprod='';
            $mtask='';
            next;
        }
        $cpic->set_prod($mprod);

        if (($mtask =~ /^(install|license)/mx) && (Cfg::opt('vxkeyless'))) {
            $mmode=$rel->prod_mode_menu($mtask,$mprod,$mmode);
            if (EDR::getmsgkey($mmode,'back')) {
                $mmode='';
                $mprod='';
                next;
            }
            $prod=$rel->prod($cpic->{prod});
            $prod->set_mode($mmode) if ($mmode && $prod->can('set_mode'));
            $moptions=$rel->prod_options_menu($mtask,$mprod,$mmode,$moptions);
            $prod->set_options($moptions) if (@{$moptions} && $prod->can('set_options'));
        } elsif ($mtask eq 'requirements') {
            $mprod=$cprod unless ($cpic->{script} =~ /^installer/m);
            $rel->prod_requirements_menu($mprod);
            $mprod='';
            $mtask='';
            next;
        } elsif ($mtask eq 'desc') {
            $rel->prod_desc_menu($mprod);
            $mprod='';
            $mtask='';
            next;
        }
        last;
    }
    return '';
}


#package Rel::UXRT601::AIX;
#
#sub define_hotfixes {
#    my $rel=shift;
#    #?? platvers "all": should be "aix61 aix71" to nix attempts on 5.x systems?
#    $rel->define_p_patches("VRTSllt60", "all", "UXRT603",  "VRTSllt");
#}
#
#package Rel::UXRT601::HPUX;             # or HPUX1131
#
#sub define_hotfixes {
#    my $rel=shift;
#    $rel->define_p_patches("VRTSllt60", "all", "UXRT603",  "PVCO_ddddd", "PVKL_ddddd");
#}

package Rel::UXRT601::RHEL5x8664;

sub define_hotfixes {
    my $rel=shift;
    #(7695) VRTSllt-6.0.300.100-GA_RHEL5.x86_64.rpm
    $rel->define_p_patches("VRTSllt60", "all", "UXRT603", "VRTSllt_6_0_300_100");
}

package Rel::UXRT601::RHEL6x8664;

sub define_hotfixes {
    my $rel=shift;
    #(7696) VRTSllt-6.0.300.100-GA_RHEL6.x86_64.rpm
    $rel->define_p_patches("VRTSllt60", "all", "UXRT603", "VRTSllt_6_0_300_100");
}

package Rel::UXRT601::SLES10x8664;

sub define_hotfixes {
    my $rel=shift;
    #(7697) VRTSllt-6.0.300.100-GA_SLES10.x86_64.rpm
    $rel->define_p_patches("VRTSllt60", "all", "UXRT603", "VRTSllt_6_0_300_100");
}

package Rel::UXRT601::SLES11x8664;

sub define_hotfixes {
    my $rel=shift;
    #(7698) VRTSllt-6.0.300.100-GA_SLES11.x86_64.rpm
    $rel->define_p_patches("VRTSllt60", "all", "UXRT603", "VRTSllt_6_0_300_100");
}
#
#package Rel::UXRT601::Sol10sparc;
#
#sub define_hotfixes {
#    my $rel=shift;
#    #(????) 012345-67
#    $rel->define_p_patches("VRTSllt60", "all", "UXRT603", "012345_67");
#}
#
#package Rel::UXRT601::Sol10x64;
#
#sub define_hotfixes {
#    my $rel=shift;
#    #(????) 123456-78
#    $rel->define_p_patches("VRTSllt60", "all", "UXRT603", "123456-78");
#}
#
#package Rel::UXRT601::Sol11sparc;
#
#sub define_hotfixes {
#    my $rel=shift;
#    #(????) VRTSllt.p5p
#    $rel->define_p_patches("VRTSllt60", "all", "UXRT603", "VRTSllt");
#}
#
#package Rel::UXRT601::Sol11x64;
#
#sub define_hotfixes {
#    my $rel=shift;
#    #(????) VRTSllt.p5p
#    $rel->define_p_patches("VRTSllt60", "all", "UXRT603", "VRTSllt");
#}


package Prod::VCS60::Common;

sub config_uuid {
    my ($prod,$result);
    $prod = shift;
    my $rel = $prod->rel;
    # Assume UUID is always correct for P-patch upgrade.
    return 1 if ($rel->{type} eq "HF");
    return -1 unless ($prod->check_uuidconfig_pl());
    $result=$prod->check_configure_uuid();
    return $result;
}

1;

=head2 LAST UPDATED

 2013-06-26 14:20	- chawley2
	Define hotfix parameter sets for VRTSllt-6.0.300.100 (Linux).

=cut

