/*
 * $Copyright: Copyright (c) 2013 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 *
 */

#ifndef _LLTAPI_H
#define _LLTAPI_H

#ifdef LINUX
static char sccsid_lltapi_h[] __attribute__((unused)) = "@(#)comms:$RCSfile: lltapi.h,v $	$Revision: 4.122 $";
#else
static char sccsid_lltapi_h[] = "@(#)comms:$RCSfile: lltapi.h,v $	$Revision: 4.122 $";
#endif

#if defined(SOLARIS) || defined(HPUX) || defined(AIX)
#ifndef LLTUNIX
#define LLTUNIX	1	/* XXX should be UNIX but already used in os.h */
#endif
#endif

#if defined(HPUX) /* { */
#ifdef _KERNEL /* { */

#include <elf.h>

#else /* _KERNEL }{ */

#if !defined(_UINT_T) && (HPUX > 1) && (HPUX < 112300)
#define _UINT_T
typedef unsigned char   uchar_t;
typedef unsigned short  ushort_t;
typedef unsigned int    uint_t;
#endif

#endif /* _KERNEL } */
#endif /* HPUX } */


#if defined(LINUX)

#ifdef _KERNEL
#include <linux/skbuff.h>
#include <linux/if.h>

#ifndef LLT_MBLK
#define LLT_MBLK
typedef struct sk_buff 	llt_mblk_t;
typedef struct sk_buff 	mblk_t;	/* clients of LLT use mblk_t. */
#endif

#else /* _KERNEL */

#include <net/if.h>
#include <inttypes.h>
#endif /* _KERNEL */

#ifdef _KMEMUSER
typedef char		llt_mblk_t;
typedef char		mblk_t;
#endif /* _KMEMUSER } */

#define LLT_MAX_DEVNAME_LEN     IFNAMSIZ
#define LLT_MAX_SYMNAME_LEN     128

#if !defined(_UINT_T)
#define _UINT_T
typedef unsigned int 	uint_t;
typedef unsigned short 	ushort_t;
typedef unsigned char 	uchar_t;
typedef unsigned long 	ulong_t;
#endif

#endif	/* defined(LINUX) */

#ifdef LLTUNIX
#include <sys/stream.h>
typedef mblk_t		llt_mblk_t;
#endif /* LLTUNIX */


#if defined (WINNT)
#include <sys/lltos.h>	//required for building GAB

#define LLT_KERNEL_PREFIX	"\\Device\\"
#define LLT_USER_PREFIX		"\\\\.\\"
#define LLT_DEVICE		LLT_USER_PREFIX "llt"
#define LLT_DEVICE_NAME		L"\\Device\\llt"

#else // UNIX

#define LLT_DEVICE		"/dev/llt"
#endif	/* defined (WINNT) */

#define LLT_NAME		"llt"

/*
 * protocol version
 */
#ifdef LLT_RU_TEST
/* when LLT is built with the -I option, this condition is hit */
#define LLT_VER_MAJ		5
#define LLT_VER_MIN		1
#else
#define LLT_VER_MAJ		5
#define LLT_VER_MIN		0
#endif

/*
 * lowest supported protocol major version
 */
#define LLT_LOWEST_VER_MAJ	5
#define LLT_LOWEST_VER_MIN	0

/* 
 * macros to compare protocol versions.
 * assumes that 
 * 	the major number is never more than 22bit long
 *	the minor number is never more than 10bit long
 */

#define LLT_MAJ_SHIFT_BITS	10

/* major * 1024 + minor; this conversion is used to make comparisons simple */
#define LLT_VER2UINT(maj, min)	\
	((uint_t)(((maj) << LLT_MAJ_SHIFT_BITS) | (min)))

#define LLT_VERSION_EQ(maj1, min1, maj2, min2)		\
	((LLT_VER2UINT((maj1), (min1))) ==		\
		(LLT_VER2UINT((maj2), (min2))))

#define LLT_VERSION_LT(maj1, min1, maj2, min2)		\
	((LLT_VER2UINT((maj1), (min1))) < 		\
		(LLT_VER2UINT((maj2), (min2))))

#define LLT_VERSION_LTE(maj1, min1, maj2, min2)		\
	((LLT_VER2UINT((maj1), (min1))) <=	 	\
		(LLT_VER2UINT((maj2), (min2))))


/* 
 * using the globals llt_vermaj and llt_vermin 
 * instead of LLT_VER_MAJ and LLT_VER_MIN 
 * because llt_vermaj can be changed at runtime 
 */
#define LLT_MAX_VER_INT	(LLT_VER2UINT(llt_vermaj, llt_vermin))

#define LLT_LOWEST_VER_INT	\
			(LLT_VER2UINT(LLT_LOWEST_VER_MAJ, LLT_LOWEST_VER_MIN))

#define LLT_VERSION_LT_SUPPORTED(maj, min)	\
	((LLT_VER2UINT((maj), (min))) < (LLT_LOWEST_VER_INT))

#define LLT_VERSION_GT_SUPPORTED(maj, min)	\
	((LLT_VER2UINT((maj), (min))) > (LLT_MAX_VER_INT))


/*
 * maximum message size in bytes not including LLT header
 * big enough to get a 64K payload with various headers
 */
#define LLT_MAX_MSGSIZE		(65 * 1024)
#define LLT_MIN_MSGSIZE		1

/*
 * Maximum number of iovecs allowed per message in llt_msgvalloc() call.
 * The value 100 depends on the max message dup 255 allowed by OS (UNIX).
 * At max 100 vectors will need 100 LLT headers and in sendv large we 
 * allocate one large mblk for headers and call dupb to get LLT header
 * for all vectors. Say if the messages need retransmission then dup 
 * count could reach 200 and rest 55 reserved for other error conditions.
 */
#define LLT_MAX_IOVCNT		100
#define LLT_MSGCNT_DEFAULT	64
#define LLT_MSGCNT_MAX		1024

/*
 * values for llt_node.n_state
 */
#define LLT_NODE_EXCLUDE	0 /* node excluded from communication */
#define LLT_NODE_IDLE		1 /* waiting for connection */
#define LLT_NODE_CONNWAIT	2 /* trying to establish connection */
#define LLT_NODE_AKWAIT		3 /* CONN_REQ,CONN_CONF xchgd, need CONN_ACK */
#define LLT_NODE_OPEN		4 /* connection established */
#define LLT_NODE_DISCON		5 /* disconnect in process */
#define LLT_NODE_INVAL		6 /* nid is out of range */

#define LLT_MAX_LINKS		8
#define LLT_MAX_TAG_LEN		12
#define LLT_MAX_MAC_LEN		16
#define LLT_MAX_USAGE_LEN	12
#define LLT_MAX_HOSTNAME	64

/*
 * reasons for CLOSENODE
 */
#define LLT_PANIC	0x0
#define LLT_TIMEOUT	0x1
#define LLT_RESET	0x2
#define LLT_SHUTDOWN	0x3
#define LLT_CLOSE	0x4
#define LLT_LINKS_REMOVED 0x5

/*
 * values for connect_func() event code
 */
#define LLT_CONNECT		0x01	/* one or more links online */
					/* reason contains number of links */
#define LLT_DISCONNECT		0x02	/* all links offline */
					/* reason contains llt_close_t */
#define LLT_DOORBELL		0x03	/* peer node wants to connect */

typedef struct llt_user_s {
	ushort_t	lu_cmd;	
	ushort_t	lu_nid;

#ifdef UNIX
	uchar_t		lu_event;
	uchar_t		lu_reason;
#endif

#if defined(LINUX) || defined(WINNT)
   	uint_t          lu_len;		/* actual len */
        uint_t          lu_buflen;	/* buffer len */
        uchar_t         *lu_buf;
#endif			/*LINUX || WINNT*/
} llt_user_t;

typedef struct llt_nodename_s {
	uint_t		hn_len;
	char		hn_name[LLT_MAX_HOSTNAME];
} llt_nodename_t;

#if defined(WINNT)
typedef struct llt_user_arp_s {
	ushort_t la_cmd;
	ushort_t la_nid;
	ushort_t la_linkno;
	uint_t la_len;
	uint_t la_buflen;
	uchar_t *la_buf;
} llt_user_arp_t;
#endif // WINNT

/*
 * on unix: values for CTL part of user-mode STREAMS requests
 * or linux, winnt : llt_user_t.lu_cmd is set to one of XXX_REQ
 * additionaly on winnt: llt_user_arp_t.la_cmd uses ARP req codes 
 *
 */
#define LLT_CONNECT_IND		0x81
#define LLT_RECV_IND		0x82
#define LLT_OPEN_REQ		0x83
#define LLT_SEND_REQ		0x84
#define LLT_DGRAM_REQ		0x85
#define LLT_BCAST_REQ		0x86
#define LLT_RECV_REQ    	0x87	/* linux, winnt */
#define LLT_SEND_ARP		0x88	/* winnt only */
#define LLT_RECV_REQ_ARP	0x89	/* winnt only */
#define LLT_RECV_REQ_ARP_START	0x90	/* winnt only */
#define LLT_RECV_REQ_ARP_END	0x91	/* winnt only */

#if defined(LINUX) || defined(WINNT)

/* parameter for IOCTL_LLT_SETPORT */
typedef struct llt_setport_s {
	uint_t	llt_port;
	uchar_t	llt_usage[LLT_MAX_USAGE_LEN];
} llt_setport_t;

#endif	/* LINUX || WINNT */

#if !defined(LLTKERNEL) && !defined(_KERNEL) /* { */
/*
 * structure for lltgethostbyname() and lltgethostbyaddr()
 */
typedef struct llt_hostent_s {
	char	*h_name;	/* name of host */
	int	h_length;	/* length of address */
	char	*h_addr;	/* LLT node ID */
} llt_hostent_t;

#ifdef WINNT	
extern void lltsethostdb(char *);
#endif
extern llt_hostent_t *lltgethostbyname(char *);
extern llt_hostent_t *lltgethostbyaddr(uint_t);

#define ERR_LLT_NOT_LOADED	(-1)
#define ERR_LLT_NOT_CONFIGURED	(-2)
extern int lltgetclustid();
extern int lltgetnodeid();
extern int lltgetnodestate();

#endif /* !_KERNEL && !LLTKERNEL } */

/*
 * record packet and byte count for last 5, 30, 60 seconds.
 */
typedef struct llt_trafstat_s {
	uint_t	ts_cnt;
	uint_t	ts_cnt1;
	uint_t	ts_cnt10;
	uint_t	ts_cnt30;	
	uint_t	ts_byt;
	uint_t	ts_byt1;
	uint_t	ts_byt10;
	uint_t	ts_byt30;	
} llt_trafstat_t;

/*
 * record flow control stats for last 1, 10, 30 seconds.
 */
typedef struct llt_flstat_s {
	uint_t	fs_cnt;
	uint_t	fs_cnt1;
	uint_t	fs_cnt10;
	uint_t	fs_cnt30;
} llt_flstat_t;

#define LLT_MAXPKTS2TRACK	32
#define LLT_MAXTRACK		32
#define LLT_POWER2BUCKETS	4
#define LLT_MAXLATBUCKETS	((1 << LLT_POWER2BUCKETS) + 1)
#define LLT_MAXBUCKETS		((1 << LLT_POWER2BUCKETS) + 1)
#define LLT_MAX_PORTS           32      /* NB: number of bits in uint */

#define LLT_MAXLINKINACTBUCKETS 6
/*
 * LLT ioctl
 */
#ifdef LLTUNIX

#define LLT_IOCTL(x)	(('L' << 8) | (x))

#define	IOCTL_LLT_GETSTATE	LLT_IOCTL(100)
#define	IOCTL_LLT_SETSTATE	LLT_IOCTL(101)
#define	IOCTL_LLT_SETPORT	LLT_IOCTL(102)
#define	IOCTL_LLT_GETPARAM	LLT_IOCTL(103)
#define	IOCTL_LLT_SETPARAM	LLT_IOCTL(104)
#define	IOCTL_LLT_SETNODERANGE	LLT_IOCTL(105)
#define	IOCTL_LLT_GETSTATS	LLT_IOCTL(106)
#define	IOCTL_LLT_CLRSTATS	LLT_IOCTL(107)
#define	IOCTL_LLT_GETNODEINFO	LLT_IOCTL(108)
#define	IOCTL_LLT_GETPORTINFO	LLT_IOCTL(109)
#define	IOCTL_LLT_GETLINKINFO	LLT_IOCTL(110)
#define	IOCTL_LLT_SETLINKINFO	LLT_IOCTL(111)
#define	IOCTL_LLT_GETNODEADDR	LLT_IOCTL(112)
#define	IOCTL_LLT_SETNODEADDR	LLT_IOCTL(113)
#define	IOCTL_LLT_DELNODEADDR	LLT_IOCTL(114)
#define	IOCTL_LLT_FLUSHNODEADDR	LLT_IOCTL(115)
#define	IOCTL_LLT_ATTACH	LLT_IOCTL(116)
#define	IOCTL_LLT_CLEAR_ATTACH	LLT_IOCTL(117)
#define	IOCTL_LLT_GETKTUNE	LLT_IOCTL(119)
#define	IOCTL_LLT_GETDRVVERS	LLT_IOCTL(120)
#define	IOCTL_LLT_GETNODEINFO_A	LLT_IOCTL(121)

#define	IOCTL_LLT_GETAPI	LLT_IOCTL(200)
#define	IOCTL_LLT_REINIT	LLT_IOCTL(201)
#define	IOCTL_LLT_DBGPRINT	LLT_IOCTL(222)
#define	IOCTL_LLT_QSTAT		LLT_IOCTL(223)
#define	IOCTL_LLT_DBGINFO	LLT_IOCTL(224)
#define	IOCTL_LLT_GETNODEALL	LLT_IOCTL(225)

#define	IOCTL_LLT_GET_PCB_QSTAT	LLT_IOCTL(226)
#define	IOCTL_LLT_GET_NCB_QSTAT	LLT_IOCTL(227)
#define IOCTL_LLT_SETMSG	LLT_IOCTL(228)
#define IOCTL_LLT_SETMSGGRP	LLT_IOCTL(229)
#define IOCTL_LLT_SETNODENAME	LLT_IOCTL(230)

#define IOCTL_LLT_TNBL_LOGGING	LLT_IOCTL(231)

#elif defined (WINNT)
/*
 * LLT DeviceIoControl
 */
#define FILE_DEVICE_LLT 0x00008123

#define IOCTL_LLT_GETAPI	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8F0, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_RELEASEAPI	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8F1, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_ATTACH	\
	CTL_CODE(FILE_DEVICE_LLT, 0x921, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_DETACH	\
	CTL_CODE(FILE_DEVICE_LLT, 0x922, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_DETACHALL	\
	CTL_CODE(FILE_DEVICE_LLT, 0x923, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_REINIT	\
	CTL_CODE(FILE_DEVICE_LLT, 0x924, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_USER		\
	CTL_CODE(FILE_DEVICE_LLT, 0x950, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_USERARP	\
	CTL_CODE(FILE_DEVICE_LLT, 0x951, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETSTATE	\
	CTL_CODE(FILE_DEVICE_LLT, 0x800, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_SETSTATE	\
	CTL_CODE(FILE_DEVICE_LLT, 0x801, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_SETPORT	\
	CTL_CODE(FILE_DEVICE_LLT, 0x802, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETPARAM	\
	CTL_CODE(FILE_DEVICE_LLT, 0x803, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_SETPARAM	\
	CTL_CODE(FILE_DEVICE_LLT, 0x804, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_SETNODERANGE	\
	CTL_CODE(FILE_DEVICE_LLT, 0x805, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETSTATS	\
	CTL_CODE(FILE_DEVICE_LLT, 0x806, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_CLRSTATS	\
	CTL_CODE(FILE_DEVICE_LLT, 0x807, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETNODEINFO	\
	CTL_CODE(FILE_DEVICE_LLT, 0x808, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETPORTINFO	\
	CTL_CODE(FILE_DEVICE_LLT, 0x809, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETLINKINFO	\
	CTL_CODE(FILE_DEVICE_LLT, 0x80A, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_SETLINKINFO	\
	CTL_CODE(FILE_DEVICE_LLT, 0x80B, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETNODEADDR	\
	CTL_CODE(FILE_DEVICE_LLT, 0x80C, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_SETNODEADDR	\
	CTL_CODE(FILE_DEVICE_LLT, 0x80D, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_DELNODEADDR	\
	CTL_CODE(FILE_DEVICE_LLT, 0x80E, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_FLUSHNODEADDR	\
	CTL_CODE(FILE_DEVICE_LLT, 0x80F, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETLINKSTAT	\
	CTL_CODE(FILE_DEVICE_LLT, 0x810, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_SETNODENAME \
	CTL_CODE(FILE_DEVICE_LLT, 0x811, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_DBGPRINT	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8F2, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_QSTAT		\
	CTL_CODE(FILE_DEVICE_LLT, 0x8F3, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_DBGINFO	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8F4, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETDBGADDR	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8F5, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETDBGDATA	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8F6, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETMSGBUF	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8F7, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETMEMSTAT	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8F8, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GET_PCB_QSTAT	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8F9, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GET_NCB_QSTAT	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8FA, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETNODEALL	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8FB, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GET_NDIS_STAT	\
	CTL_CODE(FILE_DEVICE_LLT, 0x8FC, METHOD_BUFFERED, FILE_ANY_ACCESS)
// New for WINNT 2010
#define IOCTL_LLT_SETMSG \
	CTL_CODE(FILE_DEVICE_LLT, 0x8FD, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_SETMSGGRP \
	CTL_CODE(FILE_DEVICE_LLT, 0x8FE, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETDRVVERS \
	CTL_CODE(FILE_DEVICE_LLT, 0x8FF, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_TNBL_LOGGING	\
	CTL_CODE(FILE_DEVICE_LLT, 0x900, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_LLT_GETKTUNE \
	CTL_CODE(FILE_DEVICE_LLT, 0x901, METHOD_BUFFERED, FILE_ANY_ACCESS)

#else /*LLTUNIX */ 
#define IOCTL_LLT_GETAPI	_IOR('l', 1, llt_api_t)
#define IOCTL_LLT_ATTACH	_IOWR('l', 2, llt_attach_t)
#define IOCTL_LLT_DETACH	_IOW('l', 3, llt_detach_t)
#define IOCTL_LLT_REINIT	_IO('l', 4)
#define IOCTL_LLT_USER		_IOWR('l', 5, llt_user_t)
#define IOCTL_LLT_GETSTATE	_IOWR('l', 6, uint_t)
#define IOCTL_LLT_SETSTATE	_IOW('l', 7, uint_t)
#define IOCTL_LLT_SETPORT	_IOW('l', 8, llt_setport_t)
#define IOCTL_LLT_GETPARAM	_IOWR('l', 9, llt_param_t)
#define IOCTL_LLT_SETPARAM	_IOW('l', 10, llt_param_t)
#define IOCTL_LLT_SETNODERANGE	_IOW('l', 11, llt_noderange_t)
#define IOCTL_LLT_GETSTATS	_IOR('l', 12, llt_netstat_t)
#define IOCTL_LLT_CLRSTATS	_IO('l', 13)
#define IOCTL_LLT_GETNODEINFO	_IOWR('l', 14, llt_nodeinfo_t)
#define IOCTL_LLT_GETPORTINFO	_IOWR('l', 15, llt_portinfo_t)
#define IOCTL_LLT_GETLINKINFO_PRE60	_IOWR('l', 16, llt_linkinfo_pre60_t)
#define IOCTL_LLT_SETLINKINFO_PRE60	_IOW('l', 17, llt_linkinfo_pre60_t)
#define IOCTL_LLT_GETNODEADDR	_IOWR('l', 18, llt_naddr_buf_t)
#define IOCTL_LLT_SETNODEADDR	_IOW('l', 19, llt_naddr_buf_t)
#define IOCTL_LLT_DELNODEADDR	_IOW('l', 20, llt_nodeaddr_t)
#define IOCTL_LLT_FLUSHNODEADDR	_IO('l', 21)
#define IOCTL_LLT_GETMEMSTAT	_IOR('l', 22, llt_memstat_t)
#define IOCTL_LLT_DBGPRINT	_IOW('l', 23, llt_dbgprint_t)
#define IOCTL_LLT_DBGINFO	_IOWR('l', 26, llt_dbginfo_t)
#define IOCTL_LLT_GETDBGADDR	_IOW('l', 27, int)
#define IOCTL_LLT_GETDBGDATA	_IOW('l', 28, char *)
#define IOCTL_LLT_GET_PCB_QSTAT	_IOR('l', 29, llt_pcb_qstat_t)
#define IOCTL_LLT_GET_NCB_QSTAT	_IOR('l', 30, llt_ncb_qstat_t)
#define IOCTL_LLT_GETNODEALL	_IOR('l', 31, char *)
#define IOCTL_LLT_DETACHALL	_IO('l', 32)
#define IOCTL_LLT_SETMSG	_IOW('l', 33, llt_setmsg_t)
#define IOCTL_LLT_SETMSGGRP	_IOW('l', 34, int)
#define IOCTL_LLT_GETKTUNE	_IOR('l', 36, llt_ktune_t)
#define IOCTL_LLT_GETSYMADDR	_IOWR('l', 37, llt_symaddr_t)
#define IOCTL_LLT_GETKDATA	_IOW('l', 38, llt_kdata_t)
#define IOCTL_LLT_SETNODENAME	_IOW('l', 39, llt_nodename_t)

#define IOCTL_LLT_TNBL_LOGGING	_IOWR('l', 40, char)
#define IOCTL_LLT_GETDRVVERS	_IOWR('l', 41, llt_verstr_t)
#define IOCTL_LLT_GETLINKINFO	_IOWR('l', 42, llt_linkinfo_t)
#define IOCTL_LLT_SETLINKINFO	_IOW('l', 43, llt_linkinfo_t)
#define IOCTL_LLT_GETNODEINFO_A	_IOWR('l', 44, llt_nodeinfo_t)
#endif /* LLTUNIX */

/*
 * parameter codes for GETPARAM/SETPARAM
 */
#define LP_MYNODE	1	/* rw local nid */
#define LP_MYCLUSTER	2	/* rw local cid */
#define LP_ISCONFIG	3	/* ro is local nid set? */
#define LP_VERSION_MAJ	4	/* ro major version number */
#define LP_VERSION_MIN	5	/* ro minor version number */
#define LP_HEADERSIZE	6	/* ro length of an LLT packet header */
#define LP_ATOMICMTU	7	/* ro max data length for single packet */

#define LP_LOWNODE	10	/* ro lowest valid nid */
#define LP_HIGHNODE	11	/* ro highest valid nid */
#define LP_MAXNODE	12	/* ro maximum possible nid */
#define LP_HIGHPORT	13	/* ro highest registered port */
#define LP_NUMPORT	14	/* ro count of current registered ports */
#define LP_MAXPORT	15	/* ro maximum possible port */
#define LP_MAXLINKS	16	/* ro maximum possible links [0..MAXLINKS-1] */
#define LP_NUMLINK	17	/* ro count of current links */
#define LP_MAXMSGSIZE	18	/* ro maximum message size allowed */
#define LP_BUFSZ	19	/* rw size of trace buffer (in KB) */

#define LP_HEARTBEAT	20	/* rw heartbeat timer (hipri links) */
#define LP_PEERINACT	21	/* rw peer inactivity timer */
#define LP_OOS		22	/* rw out-of-sequence timer */
#define LP_RETRANS	23	/* rw retransmit timer */
#define LP_SERVICE	24	/* rw service timer */
#define LP_HEARTBEATLO	25	/* rw heartbeat timer (lowpri links) */
#define LP_ARPEXP	26	/* rw ARP expiration timer */
#define LP_ARPSEC	27	/* rw ARPREQ expiration timer */
#define LP_NUMRESPHB	28	/* rw count of resp hbs */
#define LP_SENDHB	29	/* rw sendhb expiration timer */

#define LP_LOWAT	30	/* rw flow control low water mark */
#define LP_HIWAT	31	/* rw flow control high water mark */
#define LP_WIN		32	/* rw flow control window */
#define LP_DLPILOWAT	33	/* rw dlpi flow control low water mark (AIX) */
#define LP_DLPIHIWAT	34	/* rw dlpi flow control hi water mark (AIX) */
#define LP_LINKBURST	35	/* rw link burst size */
#define LP_UDPSNDBUF 	36      /* udp send buffer size (AIX) */
#define LP_UDPRCVBUF 	37      /* udp recieve buffer size (AIX) */
#define LP_RPORTLOWAT	38	/* rw port flow control low water mark */
#define LP_RPORTHIWAT	39	/* rw port flow control high water mark */

#define LP_NOPULLUP	45	/* rw set state packet pullup */
#define LP_DOORBELL	46	/* rw set state of DOORBELL upcalls */
#define LP_USEARP	47	/* rw set state of bcast ARP */
#define LP_BCASTHB	48	/* rw set bcast vs individual heartbeats */

#define LP_DEBUG	50	/* rw debug level */
#define LP_WARN		51	/* rw control warnings */
#define LP_XDROP	53	/* rw debug drop outgoing pkts */
#define LP_DBG_MINLINKS	54	/* rw reserved */
#define LP_CHECKSUM	55	/* rw set checksums */
#define LP_DBG_CKSM1	56	/* rw checksum test 1 */
#define LP_DBG_CKSM2	57	/* rw checksum test 2 */

#define LP_ACKVAL	58	/* rw set ack threshold */
#define LP_SWS		59	/* rw set sws */
#define LP_PEERTROUBLE	60	/* rw link inactivity */
#define LP_TRACELEVEL	61	/* rw trace level */
#define LP_STRICTSRC	62	/* rw check src address? */
#define LP_DLVIRQ	63	/* rw allow pkt delivery in interrupt ctxt */
#define LP_BROADCAST_MAJ	64	/* ro broadcast version major number */
#define LP_BROADCAST_MIN	65	/* ro broadcast version minor number */
#define LP_TIMETOREQHB	66	/* rw req hb from peer after */
#define LP_REQHBTIME	67	/* rw req hb from peer every */
#define LP_NOPUTLOCKS	68	/* rw disable streams putlocks (SOL) */
#define LP_PUTLOCKVAL	69	/* rw streams putlocks parameter (SOL) */
#define LP_MAXTHREAD	70	/* rw max threads */
#define LP_LARGEPKTLEN	71	/* rw large pkt size */
#define LP_RETRANSX	72	/* rw retransmit factor */
#define LP_CANPUTFACTOR	73	/* rw canput factor */
#define LP_MY_UNIQUE	74	/* ro canput factor */
#define LP_MAX_STALE	75	/* rw max stale pkts dropped before DISCON */
#define LP_MAX_REXMIT	76	/* rw max CONN_CONFIRM retransmits */
#define LP_MAX_CONN	77	/* rw max connect retries */
#define LP_XMITFACTOR	78	/* rw inarow factor */
#define LP_PEERTROUBLELO	79	/* rw low priority link inactivity */
#define LP_SENDHBCAP	80	/* rw cap to send hbs outside timer context */
#define LP_MYNODE_STATE	81	/* ro the local state information */
#define LP_TIMEDELAY_PANIC	82 /* rw to on/off panic on timer-delay */
#define LP_DLVBULKQ		83 /* rw deliver a queue of pkts  */
#ifdef WINNT
#define LP_MSGBUF_SIZE	84	/* ro kernel log buffer size */
#endif
#define LP_MAXIOVCNT		85 /* ro maximum iovec cnt allowed */
#define LP_LINKFAILLEVEL	86 /* rw link failure detection level */
#define LP_LINKSTABLE		87 /* rw wait before handling link failure */ 


#define DRV_VERS_STRSZ		50

/* parameter for IOCTL_LLT_GETPARAM, IOCTL_LLT_SETPARAM */
typedef struct llt_param_s {
	uint_t		lp_param;
	uint_t		lp_value;
} llt_param_t;

typedef struct llt_str_s {
	char		lp_str[DRV_VERS_STRSZ];
} llt_verstr_t;

/*
 * Framework to enable logging of System/User initiated change of the
 * LLT tunables.
 */
#define LLT_TNBL_LOG_EN		1
#define LLT_TNBL_LOG_DIS	2
#define LLT_TNBL_LOG_STAT	3

typedef struct llt_setparam_msg {
	char	*param_str;	/* String describing the tunable */
	uint_t	*value;		/* Variable which holds the tunable */
	uint_t	old_value;	/* Old Value of the tunable */
	char	enabled;	/* Enable Logging the change in tunable */
	char	set;		/* The Tunable is initially set */
} llt_setparam_msg_t;

/* parameter for IOCTL_LLT_GETPORTINFO */
typedef struct llt_portinfo_s {
	ushort_t	pi_port;
	uchar_t		pi_registered;
	uchar_t		pi_local_xoff;
	uchar_t		pi_qfull;
	uint_t		pi_qcount;
#ifdef WINNT
	void		*pi_cookie;
#else
	uint_t		pi_cookie;
#endif //WINNT
	uint_t		pi_numnodes;
	llt_trafstat_t	pi_txstat;
	uint_t		pi_xmitcnt;
	uint_t		pi_xmitbytes;
	uint_t		pi_txlatcur;
	uint_t		pi_txlatency[LLT_MAXPKTS2TRACK];
	uint_t		pi_txlatdist[LLT_MAXLATBUCKETS];
	llt_trafstat_t	pi_rxstat;
	uint_t		pi_recvcnt;
	uint_t		pi_recvbytes;
	uint_t		pi_rxlatcur;
	uint_t		pi_rxlatency[LLT_MAXPKTS2TRACK];
	uint_t		pi_rxlatdist[LLT_MAXLATBUCKETS];
	llt_flstat_t	pi_qhicnt_st;
	uint_t		pi_qhicnt;
	uint_t		pi_qlocnt;
	uint_t		pi_qhi2lotmcur;
	uint_t		pi_qhi2lotm[LLT_MAXTRACK];
	uint_t		pi_qhi2lotmdist[LLT_MAXBUCKETS];
	llt_flstat_t	pi_canputfailcnt_st;
	uint_t		pi_canputfailcnt;
	uint_t		pi_bkenablecnt;
	uint_t		pi_canputtmcur;
	uint_t		pi_canputtm[LLT_MAXTRACK];
	uint_t		pi_canputtmdist[LLT_MAXBUCKETS];
	uchar_t		pi_usage[LLT_MAX_USAGE_LEN];
	uint_t		pi_buflen;
	uchar_t		pi_nodeinfo[1];	/* one for each peer node */
	/* pi_nodeinfo flags format: */
#define LLT_CONNECTED	0x01
#define LLT_CLOSING	0x02
#define LLT_PEER_XOFF	0x04
#define LLT_OPENING	0x08
	
} llt_portinfo_t;


/*
 * struture for llt per node stat
 * current members are retransmission count for
 * nack and timeout.
 */
typedef struct llt_node_stat {
	uint_t		ns_rc_nack;	/* retransmission due to NACK */
	uint_t		ns_rc_timeout;	/* retransmission due to timeout */
} llt_nstat_t;

/* parameter for IOCTL_LLT_GETNODEINFO ( & IOCTL_LLT_GETNODEALL ) */
typedef struct llt_nodeinfo_s {
	uint_t		ni_nid;
	uint_t		ni_state;
	uint_t		ni_active_links;
	uint_t		ni_num_links;	/* min(LLT_MAX_LINKS, ni_buflen) */
	uint_t		ni_buflen;
	llt_nstat_t	ni_rstat;	/* retrans stat for node */
	uint_t		ni_pvermaj;	/* connection protocol major ver. */
	uint_t		ni_pvermin;	/* connection protocol minor ver. */
	uchar_t		ni_linkstatus[1]; /* one entry for each ni_num_links */
#define LLT_LINK_DOWN		0 /* link peerinact timer expired */
#define LLT_LINK_UP		1 /* link is receiving pkts from peer node */
#define LLT_LINK_TROUBLE	2 /* no pkts received since llt_peertrouble but 
				   * peerinact timer not expired yet.
				   * note: we report links in such state as
 				   *       LLT_LINK_UP to userspace 
				   */
	/* ni_linkstatus[1] should be the last element in this struct */
} llt_nodeinfo_t;

typedef struct llt_linkstat_s {
	uchar_t		ls_linkstatus[LLT_MAX_LINKS];
} llt_linkstat_t;

typedef struct llt_nodeall_s {
	uchar_t		*na_buf;
} llt_nodeall_t;

/* parameter for IOCTL_LLT_GETLINKINFO, IOCTL_LLT_SETLINKINFO */
typedef struct llt_linkinfo_s {
	uint_t		ll_linkno;
	uint_t		ll_flags;	/* fields to set */
#define LLT_LINKSTATE	0x01
#define LLT_LINKPRI	0x02
#define LLT_LINKBCAST	0x04
	uint_t		ll_valid;	/* ro */
	uint_t		ll_state;	/* rw */
#define LLT_DISABLE	0x00
#define LLT_RECV_ENABLE	0x01
#define LLT_XMIT_ENABLE	0x02
	uint_t		ll_pri;		/* rw */
#define LLT_HIPRI	0
#define LLT_LOWPRI	1
	uint_t		ll_linktype;		/* ro */
	uint_t		ll_mtu;			/* ro */
	ushort_t	ll_sap;			/* ro */
	uint_t		ll_sapfirst;		/* ro */
	uint_t		ll_addrlen;		/* ro */
	uint_t		ll_saplen;		/* ro */
	uint_t		ll_xmitcnt;		/* ro */
	uint_t		ll_xmitbytes;		/* ro */
	uint_t		ll_recvcnt;		/* ro */
	uint_t		ll_recvcnt_ether;	/* ro */
	uint_t		ll_recvbytes;		/* ro */
	uint_t		ll_errcnt;		/* ro */
	uint_t		ll_puthbcnt;		/* ro */
	uint_t		ll_badcksum;		/* ro */
	llt_trafstat_t	ll_txstat;		/* ro */
	llt_trafstat_t	ll_rxstat;		/* ro */
	uchar_t		ll_bcast[LLT_MAX_MAC_LEN];	/* rw */
	uchar_t		ll_tag[LLT_MAX_TAG_LEN];	/* ro */

#if defined(LLTUNIX)
	uint_t		ll_muxid;			/* ro */
#elif defined(LINUX)
	uchar_t ll_devname[LLT_MAX_DEVNAME_LEN];	/* ro */
#elif defined(WINNT)
	uint_t		ll_resets;	/* ro */
#endif
	uint_t		ll_hidden;	/* ro */
} llt_linkinfo_t;

/* parameter for IOCTL_LLT_GETLINKINFO, IOCTL_LLT_SETLINKINFO.
 * to support Rolling Upgrade the interface with VCS engine 
 * should not change so need to maintain the old structure
 * also.
 */
typedef struct llt_linkinfo_pre60_s {
	uint_t		ll_linkno;
	uint_t		ll_flags;	/* fields to set */
	uint_t		ll_valid;	/* ro */
	uint_t		ll_state;	/* rw */
	uint_t		ll_pri;		/* rw */
	uint_t		ll_linktype;		/* ro */
	uint_t		ll_mtu;			/* ro */
	ushort_t	ll_sap;			/* ro */
	uint_t		ll_sapfirst;		/* ro */
	uint_t		ll_addrlen;		/* ro */
	uint_t		ll_saplen;		/* ro */
	uint_t		ll_xmitcnt;		/* ro */
	uint_t		ll_xmitbytes;		/* ro */
	uint_t		ll_recvcnt;		/* ro */
	uint_t		ll_recvcnt_ether;	/* ro */
	uint_t		ll_recvbytes;		/* ro */
	uint_t		ll_errcnt;		/* ro */
	uint_t		ll_puthbcnt;		/* ro */
	uint_t		ll_badcksum;		/* ro */
	llt_trafstat_t	ll_txstat;		/* ro */
	llt_trafstat_t	ll_rxstat;		/* ro */
	uchar_t		ll_bcast[LLT_MAX_MAC_LEN];	/* rw */
	uchar_t		ll_tag[LLT_MAX_TAG_LEN];	/* ro */


#if defined(LLTUNIX)
	uint_t		ll_muxid;			/* ro */
#elif defined(LINUX)
	uchar_t	ll_devname[LLT_MAX_DEVNAME_LEN];	/* ro */
#endif

} llt_linkinfo_pre60_t;

/*
 * parameter for IOCTL_LLT_GETNODEADDR, IOCTL_LLT_SETNODEADDR,
 * IOCTL_LLT_DELNODEADDR
 */
typedef struct llt_nodeaddr_s {
	uint_t		ln_nid;		/* nid+linkno define unique addr */
	uint_t		ln_linkno;	/* get linkno first by matching tags */
	uint_t		ln_perm;	/* is address permanent? */
	uint_t		ln_addrlen;	/* (out) length of actual address */
	uint_t		ln_buflen;	/* (in) length of address buffer */
	uchar_t		ln_addr[1];	/* Stores hardware address */
} llt_nodeaddr_t;


/* parameter for IOCTL_LLT_SETNODERANGE */
typedef struct llt_noderange_s {
	uint_t	nr_mode;
#define	LLT_INCLUDE	0
#define	LLT_EXCLUDE	1
	uint_t		nr_min;
	uint_t		nr_max;
} llt_noderange_t;

/* parameter for IOCTL_LLT_SETMSG */
typedef struct llt_msg_s {
	uint_t		lm_min;
	uint_t		lm_max;
	uint_t		lm_val;	
} llt_setmsg_t;

/* parameter for IOCTL_LLT_GETKTUNE */
typedef struct llt_ktune_s {
	uint_t		lk_maxnodes;
	uint_t		lk_maxports;
	uint_t		lk_basetimer;

#ifdef SOLARIS
	uint_t		lk_rthreadpri;
#endif

#ifdef LINUX
	uint_t		lk_nqthread;
#endif

} llt_ktune_t;

#ifdef LINUX

/* parameter for IOCTL_LLT_GETSYMADDR */
typedef struct llt_symaddr_s {
	char		ls_symstr[LLT_MAX_SYMNAME_LEN];	/* symbol string (in) */
	uchar_t		*ls_addr;			/* address (out) */
} llt_symaddr_t;

/* parameter for IOCTL_LLT_GETKDATA */
typedef struct llt_kdata_s {
	uchar_t		*lk_srcaddr;	/* kernel addr */
	uint_t		lk_len;		/* # bytes */
	uchar_t		*lk_dstaddr;	/* userspace destination */
} llt_kdata_t;

#endif	/* LINUX */

/*
 * Networking statistics.
 * r_* refer to receive stats, x_* to xmit stats.
 *
 * this is a per-cpu structure in the driver.
 * (well actually a cache-aligned union defined below is per-cpu)
 *
 * userspace sees cumulative counts (via IOCTL_LLT_GETSTATS).
 */
typedef struct llt_netstat_s {
	uint_t	r_destdrops;	/* destid doesn't match our nid */
	uint_t	r_srcdrops;	/* src node id is out of range */
	uint_t	r_unconfdrops;	/* src node is not configured */
	uint_t	r_badgendrops;	/* connection generation num mismatch */
	uint_t	r_noconndrops;	/* node is not in connected state */
	uint_t	x_noconndrops;	/* node is not in connected state */
	uint_t	r_oowdrops;	/* out-of-window */
	uint_t	r_oospkts;	/* out-of-sequence packets */
	uint_t	r_dupdrops;	/* recv'd duplicate packets */
	uint_t	x_nobufs;	/* no mem for bufs to xmit */
	uint_t	x_retrans;	/* number of re-transmissions */
	uint_t	x_userpkts;	/* number of user pkts sent */
	uint_t	x_connpkts;	/* number of CR/CC/DR/DC pkts sent */
	uint_t	x_piggyacks;	/* number of piggybacked ACKs sent */
	uint_t	x_piggynaks;	/* number of piggybacked NAKs sent */
	uint_t	x_ackpkts;	/* number of ACK packets sent */
	uint_t	x_nakpkts;	/* number of NAK packets sent */
	uint_t	r_sapdrops;	/* rcv'd SAP is not ours */
	uint_t	r_streambad;	/* rcv'd unexpected stream primitive */
	uint_t	r_dlpibad;	/* rcv'd unexpected DLPI primitive */
	uint_t	r_dlpierr;	/* rcv'd DLPI error_ack primitive */
	uint_t	r_malformed;	/* rcv'd malformed packet */
	uint_t	r_userpkts;	/* number of user pkts received */
	uint_t	r_dgramdrop;	/* received datagrams dropped */
	uint_t	x_drops;	/* xmit drops, cant putnext to driver */
	uint_t	x_looppkts;	/* xmit loopback packets */
	uint_t	x_nolink;	/* no link to xmit packets (timed out) */
	uint_t	r_misalign;	/* Unaligned packets received */
	uint_t	r_multiblock;	/* Data received in multiple mblks */
	uint_t	r_nobufs;	/* no mem for bufs to recv */
	uint_t	r_dlpinolink;	/* no link to xmit packets (from DLPI) */
	uint_t	r_checksum;	/* bad packet checksum, dropped */
	uint_t	x_chainhdr;	/* xmit seperate header chained to payload */
	uint_t	r_badsrcaddr;	/* bad src (ether/udp) address */
	uint_t	r_oopd;		/* rcv packet with OOP data (HP) */
	uint_t	r_inseq;	/* rcv packet in-seq pkt */
	uint_t	r_lenerr;	/* rcv packet wrong length */

#ifdef LINUX
	uint_t  x_udpdrops;	/* udp xmit queue full, dropped */
#endif

} llt_netstat_t;

/*
 * error codes
 */
#ifdef LLTUNIX

#define LLT_SUCCESS	0
#define LLTERR_INVAL	EINVAL
#define LLTERR_BUSY	EBUSY
#define LLTERR_NOSUCH	ENXIO
#define LLTERR_SIZE	EMSGSIZE
#define LLTERR_CONN	ENOTCONN
#define LLTERR_NOTREG	ENOENT
#define LLTERR_ALREADY	EALREADY
#define LLTERR_IO	EIO
#define LLTERR_MEM	ENOSR
#define LLTERR_RDONLY	EROFS
#define LLTERR_FAULT	EFAULT
#define LLTERR_PERM	EPERM
#define LLTERR_FILE	EBADF
#define LLTERR_NOFILE	EMFILE
#define LLTERR_EMLINK   EMLINK

#elif defined(WINNT)	

#define LLT_SUCCESS	STATUS_SUCCESS
#define LLTERR_INVAL	STATUS_INVALID_PARAMETER
#define LLTERR_BUSY	STATUS_NETWORK_BUSY
#define LLTERR_NOSUCH	STATUS_NO_SUCH_DEVICE
#define LLTERR_SIZE	STATUS_INVALID_BUFFER_SIZE
#define LLTERR_CONN	STATUS_CONNECTION_INVALID
#define LLTERR_NOTREG	STATUS_NO_SUCH_FILE
#define LLTERR_ALREADY	STATUS_ALREADY_DISCONNECTED
#define LLTERR_IO	STATUS_NETWORK_UNREACHABLE
#define LLTERR_MEM	STATUS_NO_MEMORY
#define LLTERR_RDONLY	STATUS_MEDIA_WRITE_PROTECTED
#define LLTERR_FAULT	STATUS_ACCESS_VIOLATION
#define LLTERR_PERM	STATUS_PRIVILEGE_NOT_HELD
#define LLTERR_FILE	STATUS_INVALID_HANDLE
#define LLTERR_NOFILE	STATUS_NO_MORE_FILES

#else	/* LLTUNIX */

#define LLT_SUCCESS	0
#define LLTERR_INVAL	-EINVAL
#define LLTERR_BUSY	-EBUSY
#define LLTERR_NOSUCH	-ENXIO
#define LLTERR_SIZE	-EMSGSIZE
#define LLTERR_CONN	-ENOTCONN
#define LLTERR_NOTREG	-ENOENT
#define LLTERR_ALREADY	-EALREADY
#define LLTERR_IO	-EIO
#define LLTERR_MEM	-ENOSR
#define LLTERR_RDONLY	-EROFS
#define LLTERR_FAULT	-EFAULT
#define LLTERR_PERM	-EPERM
#define LLTERR_FILE	-EBADF
#define LLTERR_NOFILE	-EMFILE
#define LLTERR_EMLINK   -EMLINK

#endif	/* LLTUNIX */

#if defined(_KERNEL) || defined(_KMEMUSER) || defined(WINNT)/* { */

/*
 * parameter for llt_register() callbacks
 */
typedef struct llt_funcs_s {
#if defined(WINNT)
	void	(*recv_func)(void *, uint_t, llt_mblk_t *);
#else
	int	(*recv_func)(void *, uint_t, llt_mblk_t *);
#endif //WINNT
	void	(*backenable_func)(void *, uint_t);
	int	(*canput_func)(void *);
	void	(*connect_func)(void *, uint_t, int, int);
	void	(*tunable_func)(void *, llt_param_t *);
} llt_funcs_t;

/*
 * parameter for llt_msgvalloc() iovecs
 */
typedef struct llt_iovec_s {
	uchar_t		*io_bufptr;
	uint_t		io_bufsiz;
	void		(*io_freefn)(void *);
	void		*io_arg;
} llt_iovec_t;

typedef struct llt_vmsg_s {
	llt_iovec_t	*iovec;
	uint_t		iovcnt;
} llt_vmsg_t;

#endif /* _KERNEL || _KMEMUSER || WINNT } */

#if defined(_KERNEL) || (LLTKERNEL) /* { */

typedef struct llt_api {
#if defined(WINNT)
	ushort_t	lltapi_vermaj;
	ushort_t	lltapi_vermin;
	int		(*lltapi_register)();
	int		(*lltapi_unregister)();
	int		(*lltapi_opennode)();
	int		(*lltapi_closenode)();
	void		*(*lltapi_msgalloc)();
	void		*(*lltapi_bufmsgalloc)();
	void		(*lltapi_msgfree)();
	int		(*lltapi_canput)();
	int		(*lltapi_send_data)();
	void		(*lltapi_backenable)();
	int		(*lltapi_getstate)();
	int		(*lltapi_getparam)();
	int		(*lltapi_setparam)();
	int		(*lltapi_getstats)();
	int		(*lltapi_clrstats)();
	int		(*lltapi_getportinfo)();
	int		(*lltapi_getnodeinfo)();
	int		(*lltapi_getlinkinfo)();
	int		(*lltapi_setlinkinfo)();
	int		(*lltapi_getnodeaddr)();
	int		(*lltapi_setnodeaddr)();
	int		(*lltapi_delnodeaddr)();
	int		(*lltapi_flushnodeaddr)();
	int		(*lltapi_setnoderange)();
	int		(*lltapi_send_dgram)();
	int		(*lltapi_send_bcast)();
	void		(*lltapi_dumphdr)();
#else //WINNT
#ifndef LINUX
	void	*lltapi_devinfo;
#endif
	int 	(*lltapi_register)(uint_t, void *, llt_funcs_t *, uchar_t *);
	int	(*lltapi_unregister)(uint_t);
	int 	(*lltapi_opennode)(uint_t, uint_t);
	int 	(*lltapi_closenode)(uint_t, uint_t, int);
	llt_mblk_t *(*lltapi_msgalloc)(uint_t, void (*)(void));
	int	(*lltapi_canput)(uint_t, uint_t);
	int	(*lltapi_send_data)(uint_t, uint_t, llt_mblk_t *);
	void	(*lltapi_backenable)(uint_t);
	int	(*lltapi_getstate)(uint_t *);
	int	(*lltapi_setstate)(uint_t);
	int	(*lltapi_getparam)(llt_param_t *);
	int	(*lltapi_setparam)(llt_param_t *);
	int	(*lltapi_getstats)(llt_netstat_t *);
	int	(*lltapi_clrstats)(void);
	int	(*lltapi_getportinfo)(llt_portinfo_t *);
	int	(*lltapi_getnodeinfo)(llt_nodeinfo_t *, int);
	int	(*lltapi_getlinkinfo)(llt_linkinfo_t *);
	int	(*lltapi_setlinkinfo)(llt_linkinfo_t *);
	int	(*lltapi_getnodeaddr)(llt_nodeaddr_t *);
	int	(*lltapi_setnodeaddr)(llt_nodeaddr_t *);
	int	(*lltapi_delnodeaddr)(llt_nodeaddr_t *);
	int	(*lltapi_flushnodeaddr)(void);
	int	(*lltapi_setnoderange)(llt_noderange_t *);
	int	(*lltapi_send_dgram)(uint_t, uint_t, llt_mblk_t *);
	int	(*lltapi_send_bcast)(uint_t, llt_mblk_t *);
	void	(*lltapi_hold)(void);
	void	(*lltapi_release)(void);
	void	(*lltapi_dumphdr)(llt_mblk_t *);
#ifdef LINUX
	void	(*lltapi_msgfree)(struct sk_buff *);
#endif
#endif //WINNT
	llt_mblk_t *(*lltapi_msgvalloc)(uint_t, llt_vmsg_t *, uint_t);
} llt_api_t;


#if defined(SOLARIS) /* PLATFORM  { */

#define LLT_API_INIT(api) { 						\
	dev_info_t *dp; 						\
	(api) = NULL;		     					\
        if (ddi_pathname_to_dev_t("/pseudo/clone@0:llt") == (dev_t)-1) {\
		cmn_err(CE_CONT, "LLT:10069: llt not loaded\n");	\
        } else if ((dp = ddi_find_devinfo(LLT_NAME, -1, 0)) == NULL) {  \
                cmn_err(CE_CONT, "LLT:10070: llt not initialized\n");	\
        } else {     							\
		(api) = (struct llt_api *)ddi_get_driver_private(dp);	\
		if (api) {						\
			((api)->lltapi_hold)();				\
		}							\
	}     								\
}

#define LLT_API_DEINIT(api) 						\
	((api)->lltapi_release)();					\
	(api) = NULL;

#elif defined(HPUX)

#define LLT_API_INIT(api) {						\
	Elf64_Sym sym;							\
	(api) = NULL;							\
	if (mod_obj_getsymp((const char *)"llt_api_local", NULL, 0,	\
		&sym) == 0) {						\
		(api) = (llt_api_t *)sym.st_value;			\
	}								\
}

#define LLT_API_DEINIT(api) (api) = NULL

#elif defined(AIX)

#ifndef NO_VEKI

typedef struct llt_veki_api {
	ki_t		llt_ki;
	struct llt_api 	*llt_api;
} llt_veki_api_t;

#define LLTAPI_NAME	"veritas.llt"
#define LLTAPI_VERSION	"1"

#define LLT_API_INIT(api)
#define LLT_API_DEINIT(api)

#else 

extern struct llt_api llt_api_local;
#define LLT_API_INIT(api) (api) = &llt_api_local
#define LLT_API_DEINIT(api) (api) = NULL

#endif /* NO_VEKI */

#elif defined(LINUX)

#include <linux/version.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18))
extern void *llt_imc_import(char *, char *);
#define LLT_API_INIT(api) {						\
	(api) = NULL;							\
	if (!(api = (llt_api_t *)llt_imc_import("lltapi", "llt"))){	\
		printk("LLT: LLT is not loaded\n");			\
	}								\
}
#else
#define LLT_API_INIT(api)  {						\
        (api) = NULL;                                                   \
        if (!(api = (llt_api_t *)inter_module_get_request("lltapi", "llt"))){\
                printk("LLT: LLT is not loaded\n");                     \
        }                                                               \
}
#endif

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18))
extern void llt_imc_deport(char *);
#define LLT_API_DEINIT(api)	((api) = NULL, llt_imc_deport("lltapi"))
#else
#define LLT_API_DEINIT(api) 	((api) = NULL, inter_module_put("lltapi"))
#endif

#elif defined(WINNT)
extern NTSTATUS lltInitApi(PVOID *);
extern NTSTATUS lltFreeApi(PVOID);

#define LLT_API_INIT(api) lltInitApi(&api)
#define LLT_API_DEINIT(api) lltFreeApi(api)

#else
ERROR - unknown platform

#endif /* PLATFORM  } */


#define LLT_API_REGISTER(api, port, cookie, funcs, usage) \
	((api)->lltapi_register((port), (cookie), (funcs), (usage)))

#define LLT_API_UNREGISTER(api, port) \
	((api)->lltapi_unregister(port))

#define LLT_API_OPENNODE(api, port, node) \
	((api)->lltapi_opennode((port), (node)))

#define LLT_API_CLOSENODE(api, port, node, reason) \
	((api)->lltapi_closenode((port), (node), (reason)))

#define LLT_API_MSGALLOC(api, size, freefn) \
	((api)->lltapi_msgalloc((size), (freefn)))

#define LLT_API_MSGVALLOC(api, size, iovecs, cnt) \
	((api)->lltapi_msgvalloc((size), (iovecs), (cnt)))

#ifdef WINNT
#define LLT_API_BUFMSGALLOC(api, buf, size, freefn) \
	((api)->lltapi_bufmsgalloc((buf), (size), (freefn)))
#define LLT_API_MSGFREE(api, msg) \
	((api)->lltapi_msgfree((msg)))
#else //WINNT

#ifdef LINUX
#define LLT_API_MSGFREE(api, msg) \
	((api)->lltapi_msgfree(msg))
#else	/* LINUX */
#ifdef LLT_DEBUG
#define LLT_API_MSGFREE(api, msg)	{				\
						msg->b_prev = NULL;	\
						msg->b_next = NULL;	\
						freemsg(msg);		\
					}
#else	/* LLT_DEBUG */
#define LLT_API_MSGFREE(api, msg)	freemsg(msg)
#endif	/* LLT_DEBUG */
#endif	/* LINUX */
#endif //WINNT

#define LLT_API_CANPUT(api, port, node) \
	((api)->lltapi_canput((port), (node)))

#define LLT_API_SEND_DATA(api, port, node, msg) \
	((api)->lltapi_send_data((port), (node), (msg)))

#define LLT_API_BACKENABLE(api, port) \
	((api)->lltapi_backenable(port))

#define LLT_API_GETSTATE(api, state) \
	((api)->lltapi_getstate(state))

#define LLT_API_SETSTATE(api, state) \
	((api)->lltapi_setstate(state))

#define LLT_API_GETPARAM(api, param) \
	((api)->lltapi_getparam(param))

#define LLT_API_SETPARAM(api, param) \
	((api)->lltapi_setparam(param))

#define LLT_API_GETSTATS(api, stats) \
	((api)->lltapi_getstats(stats))

#define LLT_API_CLRSTATS(api) \
	((api)->lltapi_clrstats())

#define LLT_API_GETPORTINFO(api, info) \
	((api)->lltapi_getportinfo(info))

#define LLT_API_GETNODEINFO(api, info) \
	((api)->lltapi_getnodeinfo(info))

#define LLT_API_GETLINKINFO(api, info) \
	((api)->lltapi_getlinkinfo(info))

#define LLT_API_SETLINKINFO(api, info) \
	((api)->lltapi_setlinkinfo(info))

#define LLT_API_GETNODEADDR(api, addr) \
	((api)->lltapi_getnodeaddr(addr))

#define LLT_API_SETNODEADDR(api, addr) \
	((api)->lltapi_setnodeaddr(addr))

#define LLT_API_DELNODEADDR(api, addr) \
	((api)->lltapi_delnodeaddr(addr))

#define LLT_API_FLUSHNODEADDR(api) \
	((api)->lltapi_flushnodeaddr())

#define LLT_API_SETNODERANGE(api, range) \
	((api)->lltapi_setnoderange(range))

#define LLT_API_SEND_DGRAM(api, port, node, msg) \
	((api)->lltapi_send_dgram((port), (node), (msg)))

#define LLT_API_SEND_BCAST(api, port, msg) \
	((api)->lltapi_send_bcast((port), (msg)))

#define LLT_API_DUMPHDR(api, msg) \
	((api)->lltapi_dumphdr(msg))

#endif /* _KERNEL || LLTKERNEL } */

#endif	/* _LLTAPI_H */
